/**
 *
 * Copyright 2004 Hiram Chirino
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.activeio.adapter;

import java.io.IOException;

import org.activeio.Packet;
import org.activeio.RequestChannel;
import org.activeio.RequestListener;
import org.activeio.SynchChannel;


/**
 * Creates a {@see org.activeio.RequestChannel} out of a {@see org.activeio.SynchChannel}.
 * Does not support handing requests.  It can only be used to send requests.  
 * 
 * @version $Revision$
 */
final public class AsynchChannelToClientRequestChannel implements RequestChannel {
    
    private final SynchChannel next;
    
    public AsynchChannelToClientRequestChannel(SynchChannel next) {
        this.next = next;
    }

    public Packet request(Packet request, long timeout) throws IOException {
        next.write(request);
        next.flush();
        return next.read(timeout);
    }

    public void setRequestListener(RequestListener requestListener) throws IOException {
        throw new IOException("Operation not supported.");
    }

    public RequestListener getRequestListener() {
        return null;
    }

    public Object narrow(Class target) {
        return next.narrow(target);
    }

    public void dispose() {
        next.dispose();
    }

    public void start() throws IOException {
        next.start();
    }

    public void stop(long timeout) throws IOException {
        next.stop(timeout);
    }
}
